#include "vtkTestSource.h"
#include <vtkObjectFactory.h>
#include <vtkNew.h>

vtkStandardNewMacro(vtkTestSource);
vtkTestSource::vtkTestSource() : vtkPolyDataAlgorithm()
{
    this->SetNumberOfInputPorts(0);
    stepValues = new double[steps];

    for (int i = 0; i < steps; ++i)
        stepValues[i] = (double)i/(double)(steps-1);
}

vtkTestSource::~vtkTestSource()
{
    delete[] stepValues;
}

void vtkTestSource::PrintSelf(ostream& os, vtkIndent indent)
{
    this->Superclass::PrintSelf(os,indent);
}

void vtkTestSource::setPolyData(double time, vtkPolyData *data)
{
    vtkSphereSource *provider = vtkSphereSource::New();
    provider->SetRadius(10.0 * time + 1);
    provider->SetThetaResolution(18);
    provider->SetPhiResolution(18);

    data->ShallowCopy(provider->GetOutput());
    provider->Delete();
}

int vtkTestSource::RequestInformation(vtkInformation *reqInfo, vtkInformationVector **inVector, vtkInformationVector *outVector)
{
	if(this->vtkPolyDataAlgorithm::RequestInformation(reqInfo,inVector,outVector))
		return 0;

    vtkInformation *info = outVector->GetInformationObject(0);

    double tRange[2] = {stepValues[0], stepValues[steps-1]};
    info->Set(vtkStreamingDemandDrivenPipeline::TIME_RANGE(), tRange, 2);
    info->Set(vtkStreamingDemandDrivenPipeline::TIME_STEPS(), stepValues, steps);
    info->Set(vtkStreamingDemandDrivenPipeline::MAXIMUM_NUMBER_OF_PIECES(), -1);

    return 1;
}

int vtkTestSource::RequestData(vtkInformation *reqInfo, vtkInformationVector **inVector, vtkInformationVector *outVector)
{
    vtkInformation *outInfo = outVector->GetInformationObject(0);
    vtkPolyData *output = vtkPolyData::SafeDownCast(outInfo->Get(vtkDataObject::DATA_OBJECT()));

    if (!output)
        return 0;

    double time = 0.0;
    if (outInfo->Has(vtkStreamingDemandDrivenPipeline::UPDATE_TIME_STEP()))
        time = outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_TIME_STEP());

    output->Initialize();
    output->GetInformation()->Set(vtkDataObject::DATA_TIME_STEP(), time);
    this->setPolyData(time, output);

    return 1;
}

